package com.example.androidlistimages;

import android.content.ContentValues;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.provider.MediaStore;
import android.support.v4.content.CursorLoader;
import android.support.v4.widget.CursorAdapter;
import android.support.v4.widget.SimpleCursorAdapter;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.GridView;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.Toast;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;

public class MainActivity extends Activity {
	
	//define source of MediaStore.Images.Media, internal or external storage
    private final int CAPTURE_IMAGE_ACTIVITY_REQUEST_CODE = 100;
    private Uri fileUri;
	final Uri sourceUri = MediaStore.Images.Media.EXTERNAL_CONTENT_URI;
	final Uri thumbUri = MediaStore.Images.Thumbnails.EXTERNAL_CONTENT_URI;
    //final Uri thumbUri = MediaStore.Images.Thumbnails.EXTERNAL_CONTENT_URI;
	final String thumb_DATA = MediaStore.Images.Thumbnails.DATA;
	final String thumb_IMAGE_ID = MediaStore.Images.Thumbnails.IMAGE_ID;

	//SimpleCursorAdapter mySimpleCursorAdapter;
	MyAdapter mySimpleCursorAdapter;
	GridView myGridView;
    Cursor cursor;
	CursorLoader cursorLoader;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        
        myGridView = (GridView)findViewById(R.id.gridview);
        
        String[] from = {MediaStore.MediaColumns.TITLE};
        int[] to = {android.R.id.text1};

        cursorLoader = new CursorLoader(
        		this, 
        		sourceUri, 
        		null, 
        		null, 
        		null, 
        		MediaStore.Images.Media.TITLE);
        
        cursor = cursorLoader.loadInBackground();
        
        mySimpleCursorAdapter = new MyAdapter(
        		this, 
        		android.R.layout.simple_list_item_1, 
        		cursor, 
        		from, 
        		to, 
        		CursorAdapter.FLAG_REGISTER_CONTENT_OBSERVER);
        
        myGridView.setAdapter(mySimpleCursorAdapter);
        myGridView.setOnItemClickListener(myOnItemClickListener);

        this.startCameraIntent();
    }

    @Override
    protected void onStart() {
        super.onStart();
        cursor = cursorLoader.loadInBackground();
    }

    @Override
    protected void onStop() {
        super.onStop();
        if (cursor != null) cursor.close();
    }


    void startCameraIntent() {
        // give the image a name so we can store it in the phone's default location
        String timeStamp = new SimpleDateFormat("yyyyMMdd_HHmmss").format(new Date());

        ContentValues values = new ContentValues();
        values.put(MediaStore.Images.Media.TITLE, "IMG_" + timeStamp + ".jpg");

        Intent intent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);

        //fileUri = getOutputMediaFileUri(MEDIA_TYPE_IMAGE); // create a file to save the image (this doesn't work at all for images)
        fileUri = getContentResolver().insert(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, values); // store content values
        intent.putExtra( MediaStore.EXTRA_OUTPUT, fileUri);

        // start the image capture Intent
        startActivityForResult(intent, CAPTURE_IMAGE_ACTIVITY_REQUEST_CODE);
    }

    OnItemClickListener myOnItemClickListener = new OnItemClickListener(){
		@Override
		public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
			Cursor cursor = mySimpleCursorAdapter.getCursor();
			cursor.moveToPosition(position);
            String data = cursor.getString(cursor.getColumnIndex(MediaStore.Images.Media.DATA));
			getThumbnail(data);
		}
    };
	
	private Bitmap getThumbnail(String srcPath){

        Bitmap sourceBitmap = null;
        if(srcPath != null){
            Toast.makeText(getApplicationContext(), srcPath, Toast.LENGTH_LONG).show();
            BitmapFactory.Options bmpFactoryOptions = new BitmapFactory.Options();
            bmpFactoryOptions.inSampleSize = 2;
            sourceBitmap = BitmapFactory.decodeFile(srcPath, bmpFactoryOptions);
            this.showDialogWithImage(sourceBitmap);
        }else{
            Toast.makeText(getApplicationContext(), "NO Source Image!", Toast.LENGTH_LONG).show();
        }

        return sourceBitmap;

	}
	
	public class MyAdapter extends SimpleCursorAdapter{
		
		Cursor myCursor;
	    Context myContext;

		public MyAdapter(Context context, int layout, Cursor c, String[] from,
				int[] to, int flags) {
			super(context, layout, c, from, to, flags);
			
			myCursor = c;
		    myContext = context;
            myCursor.moveToPosition(0);
		}

		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			View row = convertView;
			if(row==null){
				LayoutInflater inflater=getLayoutInflater();
				row=inflater.inflate(R.layout.row, parent, false);	
			}
			
			ImageView thumbV = (ImageView)row.findViewById(R.id.thumb);
			
			myCursor.moveToPosition(position);
			
			int myID = myCursor.getInt(myCursor.getColumnIndex(MediaStore.Images.Media._ID));
			String[] srcColumns = {MediaStore.Images.Media.DATA, MediaStore.Images.Media._ID};

			CursorLoader sourceCursorLoader = new CursorLoader(
					myContext, 
	        		sourceUri,
					srcColumns,
                    MediaStore.Images.Media._ID + "=" + myID,
					null, 
					null);

			Cursor sourceCursor = sourceCursorLoader.loadInBackground();
			
			Bitmap myBitmap = null;
	        if(sourceCursor.moveToFirst()){
				int srcColumnIndex = sourceCursor.getColumnIndex(MediaStore.Images.Media.DATA);
				String srcPath = sourceCursor.getString(srcColumnIndex);
                BitmapFactory.Options bmpFactoryOptions = new BitmapFactory.Options();
                bmpFactoryOptions.inSampleSize = 4;
                myBitmap = BitmapFactory.decodeFile(srcPath, bmpFactoryOptions);
				thumbV.setImageBitmap(myBitmap);
			}

			return row;
		}
	}

    /**
     * Camera intent receiver
     * @param requestCode
     * @param resultCode
     * @param data
     */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (resultCode == RESULT_OK && requestCode == CAPTURE_IMAGE_ACTIVITY_REQUEST_CODE) {
            BitmapFactory.Options bmpFactoryOptions = new BitmapFactory.Options();
            bmpFactoryOptions.inSampleSize = 2;
            String bmpPath = fileUri.getPath();
            String encodedPath = fileUri.getEncodedPath();

            String filePath = null;
            if (fileUri != null) {
                Cursor cursor = this.getContentResolver().query(fileUri, new String[] { android.provider.MediaStore.Images.ImageColumns.DATA }, null, null, null);
                cursor.moveToFirst();
                filePath = cursor.getString(0);
                cursor.close();
            } else {
                filePath = fileUri.getPath();
            }
            Bitmap sourceBitmap = BitmapFactory.decodeFile(filePath, bmpFactoryOptions);
            this.showDialogWithImage(sourceBitmap);

        }
        super.onActivityResult(requestCode, resultCode, data);
    }

    void showDialogWithImage(Bitmap bitmap) {
        AlertDialog.Builder thumbDialog = new AlertDialog.Builder(MainActivity.this);
        ImageView thumbView = new ImageView(MainActivity.this);
        thumbView.setImageBitmap(bitmap);
        LinearLayout layout = new LinearLayout(MainActivity.this);
        layout.setOrientation(LinearLayout.VERTICAL);
        layout.addView(thumbView);
        thumbDialog.setView(layout);
        thumbDialog.show();
    }
}
